//
//  QAAppDelegate.m
//  TexturePainting
//
//  Created by Daniel Dönigus on 14.10.12.
//  Copyright (c) 2012 Quality Applications. All rights reserved.
//

#import "QAAppDelegate.h"

@implementation QAAppDelegate

#pragma mark - initialization

- (void) createSceneWithLightAndCamera {
    self.sceneView.scene = [SCNScene scene];
    
    //Create node with light and camera
    SCNNode* cameraLightNode = [SCNNode node];
    cameraLightNode.camera = [SCNCamera camera];
    cameraLightNode.light = [SCNLight light];
    cameraLightNode.position = SCNVector3Make(0.0, 0.0, 2.5);
    cameraLightNode.light.color = [NSColor whiteColor];
    cameraLightNode.light.type = SCNLightTypeOmni;
    [self.sceneView.scene.rootNode addChildNode:cameraLightNode];
}

- (void) createGeometryWithLayerTexture {
    self.sceneView.scene = [SCNScene scene];
    
    //Create geometry and node
    SCNNode* geometryNode = [SCNNode node];
    geometryNode.geometry = [SCNSphere sphereWithRadius:1.0];
    geometryNode.rotation = SCNVector4Make(0.0, 1.0, 0.0, M_PI_4);
    [self.sceneView.scene.rootNode addChildNode:geometryNode];
    
    //Create layer
    CALayer* textureLayer = [CALayer layer];
    textureLayer.bounds = CGRectMake(0, 0, 1024, 1024);
    textureLayer.frame = CGRectMake(0, 0, 1024, 1024);
    [textureLayer setBackgroundColor:[[NSColor whiteColor] CGColor]];
    textureLayer.delegate = self;
    
    //Create material with layer as contents
    SCNMaterial* material = [SCNMaterial material];
    material.diffuse.contents = textureLayer;
    geometryNode.geometry.firstMaterial = material;
    
    self.textureLayer = textureLayer;
}


- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    self.geometry = [NSMutableArray array];
    [self createSceneWithLightAndCamera];
    [self createGeometryWithLayerTexture];
}

#pragma mark - Layer delegate

- (void) paintGeometry:(QAGeometry*) geometry forLayer:(CALayer*) layer inContext:(CGContextRef)context {
    //Paint geometry in context by moving to first point and drawing lines to each point in geometry class
    //The points in geometry class are in range 0-1, so we have to multiply by layer.bounds
    CGPoint cgStartPoint = [[geometry.points objectAtIndex:0] pointValue];
    CGContextMoveToPoint(context, cgStartPoint.x*layer.bounds.size.width, cgStartPoint.y*layer.bounds.size.height);
    
    for (NSValue* point in geometry.points) {
        CGPoint cgPoint = [point pointValue];
        
        CGContextAddLineToPoint(context, cgPoint.x*layer.bounds.size.width, cgPoint.y*layer.bounds.size.height);
    }
    
    CGContextStrokePath(context);
}

- (void)drawLayer:(CALayer *)layer inContext:(CGContextRef)context {
    CGContextSetStrokeColorWithColor(context, [[NSColor blackColor] CGColor]);
    
    //Draw current drawing geometry
    if (self.additionalGeometry != nil) {
        [self paintGeometry:self.additionalGeometry forLayer:layer inContext:context];
    }
    
    //Draw geometry in array
    for (QAGeometry* geometry in self.geometry) {
        [self paintGeometry:geometry forLayer:layer inContext:context];
    }
}

#pragma mark - Handling drawing geometry

- (void) drawingStartedAtPosition:(CGPoint) point {
    //Creating a new additional geometry
    self.additionalGeometry = [[QAGeometry alloc] init];
    //Adding first point to geometry
    [self.additionalGeometry.points addObject:[NSValue valueWithPoint:point]];
    
    //Refresh layers
    [self.textureLayer setNeedsDisplay];
    [self.viewLayer setNeedsDisplay];
}

- (void) addAdditionalPointAtPosition:(CGPoint) point {
    //Adding additional point to current drawing geometry
    [self.additionalGeometry.points addObject:[NSValue valueWithPoint:point]];

    //Refresh layers
    [self.textureLayer setNeedsDisplay];
    [self.viewLayer setNeedsDisplay];
}

- (void) finalizeGeometry {
    //Add current drawing geometry to array
    [self.geometry addObject:self.additionalGeometry];
    self.additionalGeometry = nil;
    
    //Refresh layers
    [self.textureLayer setNeedsDisplay];
    [self.viewLayer setNeedsDisplay];
}

@end
